/*
 [The "BSD licence"]
 Copyright (c) 2005 Jean Bovet, http://www.cs.usfca.edu/~jbovet/
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
														   NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
														   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "EEditorView.h"

#import "ENSTextView.h"
#import "ENSScroller.h"

#import "EthernetPacket.h"

#import "Constants.h"

@interface EEditorView (PrivateMethods)
- (void)createAddrView:(NSRect)frame;
- (void)createHexaView:(NSRect)frame;
- (void)createAsciiView:(NSRect)frame;
@end

@implementation EEditorView

- (id)initWithFrame:(NSRect)frame
{
    if (self = [super initWithFrame:frame]) {
		mSelectionLooping = NO;
		mCharactersWidth = 30;
		mPacket = NULL;
		[self createAddrView:frame];
		[self createHexaView:frame];
		[self createAsciiView:frame];
		[self setNeedsDisplay:YES];
	}
    return self;
}

- (BOOL)isFlipped
{
	return YES;
}

- (float)addrWidth
{
	NSMutableDictionary *attrs = [NSMutableDictionary dictionary];
	[attrs setObject:[NSFont fontWithName:@"Courier" size:12] forKey:NSFontAttributeName];
	NSSize textSize = [@"  9999:" sizeWithAttributes:attrs];
	return textSize.width;	
}

- (float)addrHeight
{
	NSMutableDictionary *attrs = [NSMutableDictionary dictionary];
	[attrs setObject:[NSFont fontWithName:@"Courier" size:12] forKey:NSFontAttributeName];
	NSSize textSize = [@" jI," sizeWithAttributes:attrs];
	return textSize.height*[mPacket numberOfLineForWidth:mCharactersWidth];	
}

- (float)hexaWidth
{
	NSMutableDictionary *attrs = [NSMutableDictionary dictionary];
	[attrs setObject:[NSFont fontWithName:@"Courier" size:12] forKey:NSFontAttributeName];
	NSSize textSize = [@"A" sizeWithAttributes:attrs];
	return textSize.width*(mCharactersWidth*CHAR_PER_HEX+1);
}

- (float)asciiWidth
{
	NSMutableDictionary *attrs = [NSMutableDictionary dictionary];
	[attrs setObject:[NSFont fontWithName:@"Courier" size:12] forKey:NSFontAttributeName];
	NSSize textSize = [@"A" sizeWithAttributes:attrs];
	return textSize.width*(mCharactersWidth+1);
}

- (void)createAddrView:(NSRect)frame
{	
	mAddrView = [[ENSTextView alloc] initWithFrame:NSMakeRect(0, 0, [self addrWidth], frame.size.height)];
	[mAddrView setFont:[NSFont fontWithName:@"Courier" size:12]];
	[mAddrView setContinuousSpellCheckingEnabled:NO];
	[mAddrView setDelegate:self];	
	[mAddrView setEditable:NO];
	[mAddrView setSelectable:NO];
	
	[self addSubview:mAddrView];	
	
	[mAddrView setString:@""];
}

- (void)createHexaView:(NSRect)frame
{	
	mHexaView = [[ENSTextView alloc] initWithFrame:NSMakeRect([self addrWidth], 0, [self hexaWidth], frame.size.height)];
	[mHexaView setFont:[NSFont fontWithName:@"Courier" size:12]];
	[mHexaView setContinuousSpellCheckingEnabled:NO];
	[mHexaView setDelegate:self];	
	[mHexaView setEditable:NO];
	[mHexaView setUsesFindPanel:YES];

	[self addSubview:mHexaView];		
}

- (void)createAsciiView:(NSRect)frame
{	
	mAsciiView = [[ENSTextView alloc] initWithFrame:NSMakeRect([self addrWidth]+[self hexaWidth], 0, frame.size.width-([self addrWidth]+[self hexaWidth]), frame.size.height)];
	[mAsciiView setFont:[NSFont fontWithName:@"Courier" size:12]];		
	[mAsciiView setContinuousSpellCheckingEnabled:NO];		
	[mAsciiView setDelegate:self];	
	[mAsciiView setEditable:NO];
	[mAsciiView setUsesFindPanel:YES];
	
	[self addSubview:mAsciiView];
}

- (void)resize
{
	NSSize size = [mHexaView frame].size;
	size.width = [self hexaWidth];
	size.height = [self addrHeight];
	[mHexaView setFrameSize:size];
	
	NSRect frame = [mAsciiView frame];
	frame.origin.x = [self addrWidth]+[self hexaWidth];
	frame.size.width = [self asciiWidth];
	frame.size.height = [self addrHeight];
	[mAsciiView setFrame:frame];
		
	[self setFrameSize:NSMakeSize([self addrWidth]+[self hexaWidth]+[self asciiWidth], frame.size.height)];
	[self setNeedsDisplay:YES];
}

- (void)setCharactersWidth:(int)width
{
	mCharactersWidth = width;
	[self resize];
}

- (void)setContent:(EthernetPacket*)result
{
	mPacket = result;
	
	if(result == NULL) {
		[mAddrView setString:@""];
		[mHexaView setString:@""];
		[mAsciiView setString:@""];
	} else {		
		[mAddrView setString:[result addrWidth:mCharactersWidth]];
		[mHexaView setString:[result hexaWidth:mCharactersWidth]];
		[mAsciiView setString:[result asciiWidth:mCharactersWidth]];		
	}

	[self resize];
}

- (void)setHighlightString:(NSString*)string
{
	NSString *t = [mAsciiView string];
	NSRange r = NSMakeRange(0, [t length]);
	
	[[mAsciiView textStorage] removeAttribute:NSBackgroundColorAttributeName range:r];
	
	while(YES) {
		r = [t rangeOfString:string options:NSCaseInsensitiveSearch range:r];
		if(r.location == NSNotFound)
			break;
		
		[[mAsciiView textStorage] addAttribute:NSBackgroundColorAttributeName value:[NSColor redColor] range:r];
		
		r.location += r.length;
		if(r.location >= [t length])
			break;
		r.length = [t length]-r.location;
	}
}

// NSTextView delegate

- (void)textView:(NSTextView *)tv selectionRangeForProposedRange:(NSRange*)r
{
	if(tv == mHexaView) {
		// Adjust beginning of range to first hexa digit
		
		int max = [[tv string] length];
		
		if(r->location>0) {
			int location = r->location-1;
			while([[tv string] characterAtIndex:location] != ' ' && location > 0) {
				location--;
			}
			
			if(location != 0)
				location++;
			
			r->length += r->location-location;
			r->location = location;
		}		
		
		// Adjust end of range to white space after hexa digits

		int location = r->location+r->length+1;
		if(location<max) {
			while([[tv string] characterAtIndex:location] != ' ' && location < max) {
				location++;
			}
			if(location < max)
				location++;
			r->length = location-r->location;
		}		
	}
}

- (NSRange)textView:(NSTextView *)tv willChangeSelectionFromCharacterRange:(NSRange)or toCharacterRange:(NSRange)nr
{
	if(mSelectionLooping)
		return nr;
	mSelectionLooping = YES;
	
	NS_DURING
		if(tv == mHexaView) {
			int cr_count = nr.location/(CHAR_PER_HEX*mCharactersWidth+1);
			int start = (nr.location-cr_count)/CHAR_PER_HEX+cr_count;

			int end = nr.location+nr.length-1;
			cr_count = (nr.location+nr.length-1)/(CHAR_PER_HEX*mCharactersWidth+1);
			end = (end-cr_count)/CHAR_PER_HEX+cr_count;
						
			NSRange asciiRange = NSMakeRange(start, end-start+1);
			if(asciiRange.location+asciiRange.length <= [[mAsciiView string] length])
				[mAsciiView setSelectedRange:asciiRange];
		} else if(tv == mAsciiView) {
			int cr_count = nr.location/(mCharactersWidth+1);
			int start = (nr.location-cr_count)*CHAR_PER_HEX+cr_count;

			int end = nr.location+nr.length;
			cr_count = end/(mCharactersWidth+1);
			end = (end-cr_count)*CHAR_PER_HEX-1+cr_count;
			
			NSRange hexaRange = NSMakeRange(start, end-start+1);
			if(hexaRange.location+hexaRange.length <= [[mHexaView string] length])
				[mHexaView setSelectedRange:hexaRange];		
		}
	NS_HANDLER
	NS_ENDHANDLER
	
	mSelectionLooping = NO;
	
	return nr;
}

// ENSScroller delegate

- (void)scroller:(NSScroller*)scroller setFloatValue:(float)aFloat knobProportion:(float)knobProp
{
//	[mAddrView scrollRectToVisible:[mAsciiView visibleRect]];
//	[mHexaView scrollRectToVisible:[mAsciiView visibleRect]];
}

@end
