/*
 [The "BSD licence"]
 Copyright (c) 2005 Jean Bovet, http://www.cs.usfca.edu/~jbovet/
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
														   NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
														   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "PacketSender.h"
#import "Ethernal.h"

#import "GetEthernetAddrSample.h"

@interface PacketSender (PrivateMethods)
- (void)updateTextInfo;
- (NSString*)text;
@end

@implementation PacketSender

- (id)init
{
	if(self = [super init]) {
		mSlots = [[NSMutableArray alloc] init];
	}
	return self;
}

- (void)dealloc
{
	[mSlots release];
	[super dealloc];
}

- (void)awakeFromNib
{
	[mSlotPopUp removeAllItems];
	
	EnetData data[10];
	UInt32 entryCount = 10;
	int err = GetEthernetAddressInfo((EnetData*)&data, &entryCount);
	if(err == noErr) {
		int i;
		for(i=0; i<entryCount; i++) {
			NSMutableString *title = [NSMutableString string];
			[title appendFormat:@"%s", data[i].bsdName];
			if(data[i].isBuiltIn)
				[title appendString:@" Built-in"];
			[mSlotPopUp addItemWithTitle:title];
			//if(i == entryCount-1)
			//	[mSlotPopUp setStringValue:title];
			
			NSMutableDictionary *dic = [NSMutableDictionary dictionary];
			[dic setObject:title forKey:@"title"];
			[dic setObject:[NSString stringWithFormat:@"%s", data[i].bsdName] forKey:@"slot"];
			[dic setObject:[NSValue valueWithBytes:&data[i].macAddress objCType:@encode(MACAddress)] forKey:@"MAC"];
			[mSlots addObject:dic];
		}
	}
	
	[self updateTextInfo];
}

- (void)updateTextInfo
{
	[mTextInfoField setStringValue:[NSString stringWithFormat:@"Length = %d characters", [[self text] length]]];
}

- (void)textDidChange:(NSNotification *)notif
{
	[self updateTextInfo];
}

- (NSString*)slot
{
	return [[mSlots objectAtIndex:[mSlotPopUp indexOfSelectedItem]] objectForKey:@"slot"];
}

- (NSString*)text
{
	return [[NSUserDefaults standardUserDefaults] objectForKey:@"data_text"];
}

- (BOOL)dataText
{
	return [[[NSUserDefaults standardUserDefaults] objectForKey:@"data_type"] intValue] == 0;	
}

- (unsigned)dataLength
{
	if([self dataText])
		return [[self text] length];
	else
		return [[[NSUserDefaults standardUserDefaults] objectForKey:@"data_length"] intValue];
}

- (NSString*)destinationMAC
{
	return [[NSUserDefaults standardUserDefaults] objectForKey:@"dest_MAC"];
}

- (NSString*)sourceMAC
{
	return [[NSUserDefaults standardUserDefaults] objectForKey:@"source_MAC"];
}

- (unsigned)type
{
	NSString *typeString = [[NSUserDefaults standardUserDefaults] objectForKey:@"type"];
	if(typeString) {
		NSScanner *scanner = [NSScanner scannerWithString:typeString];
		unsigned int type;
		[scanner scanHexInt:&type];
		return type;		
	} else {
		return 0;
	}
}

- (void)getMAC:(unsigned char*)mac fromString:(NSString*)smac
{
	if(smac) {
		NSScanner *scanner = [NSScanner scannerWithString:smac];
		int i;
		for(i=0; i<6; i++) {
			unsigned int value;
			[scanner scanHexInt:&value];
			unsigned char v = (unsigned char)value;
			memcpy(&mac[i], &v, 1);
			if(i < 5)
				[scanner scanString:@":" intoString:NULL];
		}				
	}
}

- (int)packetCount
{
	return [[[NSUserDefaults standardUserDefaults] objectForKey:@"packet_count"] intValue];
}

- (double)packetDelay
{
	return (double)([[[NSUserDefaults standardUserDefaults] objectForKey:@"packet_delay"] intValue]/1000);
}

- (IBAction)getCurrentMAC:(id)sender
{
	NSDictionary *info = [mSlots objectAtIndex:[mSlotPopUp indexOfSelectedItem]];
	MACAddress mac;
	NSValue *value = [info objectForKey:@"MAC"];
	[value getValue:&mac];
	NSMutableString *s = [NSMutableString string];
	int i;
	for(i=0; i<kIOEthernetAddressSize; i++) {
		[s appendFormat:@"%02X", mac[i]];
		if(i < kIOEthernetAddressSize-1)
			[s appendString:@":"];
	}
	[[NSUserDefaults standardUserDefaults] setObject:s forKey:@"source_MAC"];
}

- (IBAction)send:(id)sender
{
	typedef struct	{
		unsigned char	dst[6];
		unsigned char	src[6];
		unsigned short	typlen;
		unsigned short	nbytes;
	} ethernet_header_struct;
	
	ethernet_header_struct header = { 0	};
	
	[self getMAC:&header.dst[0] fromString:[self destinationMAC]];
	[self getMAC:&header.src[0] fromString:[self sourceMAC]];
	
	header.typlen = htons([self type]);
	header.nbytes = htons([self dataLength]);
		
	int packet_length = sizeof(header)+header.nbytes;
	char *packet = (char*)malloc(packet_length);
	memcpy(packet, &header, sizeof(header));
	
	if([self dataText]) {		
		memcpy(packet+sizeof(header), [[self text] lossyCString], header.nbytes);
	} else {
		int i;
		for(i=0; i<header.nbytes; i++) {
			memset(packet+sizeof(header)+i, rand()%0xFF, 1);
		}		
	}
	
	int socket = [[Ethernal shared] getSocketForSlot:[self slot]];
	if(socket >= 0) {
		int i;
		for(i=0; i<[self packetCount]; i++) {
			write(socket, packet, packet_length);
			[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:[self packetDelay]]];
		}
	}
	
	free(packet);
}

@end
